@@ -36,11 +36,11 @@ RDF::LinkedData - A simple Linked Data implementation
 
 =head1 VERSION
 
-Version 0.49_1
+Version 0.55_1
 
 =cut
 
-our $VERSION = '0.49_1';
+our $VERSION = '0.55_1';
 
 
 =head1 SYNOPSIS
@@ -121,13 +121,15 @@ sub BUILD {
  	if ($self->has_void_config) {
 		$self->logger->debug('VoID config found with parameters: ' . Dumper($self->void_config) );
 
-		unless (can_load( modules => { 'RDF::Generator::Void' => 0.02 })) {
+		unless (can_load( modules => { 'RDF::Generator::Void' => 0.04 })) {
 			throw Error -text => "RDF::Generator::Void not installed. Please install or remove its configuration.";
 		}
 		my $dataset_uri = (defined($self->void_config->{dataset_uri}))
 								  ? $self->void_config->{dataset_uri} 
 								  : URI->new($self->base_uri . '#dataset-0')->canonical;
-		$self->void(RDF::Generator::Void->new(inmodel => $self->model, dataset_uri => $dataset_uri));
+		$self->void(RDF::Generator::Void->new(inmodel => $self->model, 
+														  dataset_uri => $dataset_uri,
+														  namespaces_as_vocabularies => $self->void_config->{namespaces_as_vocabularies}));
  	} else {
 		$self->logger->info('No VoID config found');
 	}
@@ -142,7 +144,8 @@ Returns the RDF::Trine::Model object.
 
 =cut
 
-has model => (is => 'ro', isa => 'RDF::Trine::Model', lazy => 1, builder => '_build_model', handles => ['etag']);
+has model => (is => 'ro', isa => 'RDF::Trine::Model', lazy => 1, builder => '_build_model', 
+				  handles => { current_etag => 'etag' });
 
 sub _build_model {
 	my $self = shift;
@@ -187,9 +190,18 @@ Returns the L<Plack::Request> object if it exists or sets it if a L<Plack::Reque
 has request => ( is => 'rw', isa => 'Plack::Request');
 
 
-=item C<< etag >>
+=item C<< current_etag >>
+
+Returns the current Etag of the model suitable for use in a HTTP header. This is a read-only attribute.
+
+=item C<< last_etag >>, C<< has_last_etag >>
+
+Returns or sets the last Etag of so that changes to the model can be detected.
+
+=cut
+
+has last_etag => ( is => 'rw', isa => 'Str', predicate => 'has_last_etag');
 
-Returns an Etag suitable for use in a HTTP header
 
 =item namespaces ( { skos => 'http://www.w3.org/2004/02/skos/core#', dct => 'http://purl.org/dc/terms/' } )
 
@@ -257,9 +269,11 @@ sub response {
 			$response->status(200);
 			my $content = $self->_content($node, $type, $endpoint_path);
 			$response->headers->header('Vary' => join(", ", qw(Accept)));
-			$response->headers->header('ETag' => $self->etag);
+			if (defined($self->current_etag)) {
+				$response->headers->header('ETag' => $self->current_etag);
+			}
 			$response->headers->content_type($content->{content_type});
-			$response->content($content->{body});
+			$response->body(encode_utf8($content->{body}));
 		} else {
 			$response->status(303);
 			my ($ct, $s) = $self->_negotiate($headers_in);
@@ -367,22 +381,28 @@ sub _content {
 		$output{content_type} = $ctype;
 		if ($self->hypermedia) {
 			my $hmmodel = RDF::Trine::Model->temporary_model;
-			if($self->has_endpoint) {
+			if($self->has_void) {
 				$hmmodel->add_statement(statement(iri($node->uri_value . '/data'), 
 															 iri('http://rdfs.org/ns/void#inDataset'), 
-															 blank('void')));
-				$hmmodel->add_statement(statement(blank('void'), 
-															 iri('http://rdfs.org/ns/void#sparqlEndpoint'),
-															 iri($self->base_uri . $endpoint_path)));
-			}
-			if($self->namespaces_as_vocabularies) {
-				$hmmodel->add_statement(statement(iri($node->uri_value . '/data'), 
-															 iri('http://rdfs.org/ns/void#inDataset'), 
-															 blank('void')));
-				foreach my $nsuri (values(%{$self->namespaces})) {
+															 $self->void->dataset_uri));
+			} else {
+				if($self->has_endpoint) {
+					$hmmodel->add_statement(statement(iri($node->uri_value . '/data'), 
+																 iri('http://rdfs.org/ns/void#inDataset'), 
+																 blank('void')));
 					$hmmodel->add_statement(statement(blank('void'), 
-																 iri('http://rdfs.org/ns/void#vocabulary'),
-																 iri($nsuri)));
+																 iri('http://rdfs.org/ns/void#sparqlEndpoint'),
+																 iri($self->base_uri . $endpoint_path)));
+				}
+				if($self->namespaces_as_vocabularies) {
+					$hmmodel->add_statement(statement(iri($node->uri_value . '/data'), 
+																 iri('http://rdfs.org/ns/void#inDataset'), 
+																 blank('void')));
+					foreach my $nsuri (values(%{$self->namespaces})) {
+						$hmmodel->add_statement(statement(blank('void'), 
+																	 iri('http://rdfs.org/ns/void#vocabulary'),
+																	 iri($nsuri)));
+					}
 				}
 			}
 			$iter = $iter->concat($hmmodel->as_stream);
@@ -401,8 +421,8 @@ sub _content {
 														  title => $preds->title( $node ),
 														  base => $self->base_uri,
 														  namespaces => $self->namespaces);
-		my $writer = HTML::HTML5::Writer->new( markup => 'xhtml', doctype => DOCTYPE_XHTML_RDFA );
-		$output{body} = encode_utf8( $writer->document($gen->create_document($returnmodel)) );
+		my $writer = HTML::HTML5::Writer->new( charset => 'ascii', markup => 'html' );
+		$output{body} = $writer->document($gen->create_document($returnmodel));
 		$output{content_type} = 'text/html';
 	}
 	return \%output;
@@ -446,7 +466,7 @@ sub _negotiate {
 																	 namespaces => $self->namespaces,
 																	 extend => {
 																					'text/html' => 'html',
-																					'application/xhtml+xml' => 'html'
+																					'application/xhtml+xml' => 'xhtml'
 																				  }
 																	)
 	};
@@ -468,37 +488,87 @@ sub _void_content {
 	my $fragment = $dataset_uri->fragment;
 	$dataset_uri =~ s/(\#$fragment)$//;
 	if ($uri->eq($dataset_uri)) {
-		$generator->urispace($self->base_uri);
-		if ($self->has_endpoint) {
-			$generator->add_endpoints($self->base_uri . $endpoint_path);
+
+		# First check if the model has changed, the etag will have
+		# changed, and we will have to regenerate at some point. If
+		# there is no current etag, we clear anyway
+		if ((! defined($self->current_etag)) || ($self->has_last_etag && ($self->last_etag ne $self->current_etag))) {
+			$self->_clear_voidmodel; 
 		}
-		my $voidmodel = $generator->generate;
+
+
+		# Now really regenerate if there is no model now
+	   unless ($self->_has_voidmodel) {
+
+			# First see if we should read some static stuff from file
+			my $file_model = undef;
+			if ($self->void_config->{add_void}) {
+				$file_model = RDF::Trine::Model->temporary_model;
+				my $parser = RDF::Trine::Parser->new($self->void_config->{add_void}->{syntax});
+				$parser->parse_file_into_model($self->base_uri, $self->void_config->{add_void}->{file}, $file_model);
+			}
+
+			# Use the methods of the generator to add stuff from config, etc
+			if ($self->void_config->{urispace}) {
+				$generator->urispace($self->void_config->{urispace});
+			} else {
+				$generator->urispace($self->base_uri);
+			}
+			if ($self->namespaces_as_vocabularies) {
+				$generator->add_vocabularies(values(%{$self->namespaces}));
+			}
+			if ($self->has_endpoint) {
+				$generator->add_endpoints($self->base_uri . $endpoint_path);
+			}
+			if ($self->void_config->{licenses}) {
+				$generator->add_licenses($self->void_config->{licenses});
+			}
+			foreach my $title (@{$self->void_config->{titles}}) {
+				$generator->add_titles(literal(@{$title}));
+			}
+			if ($self->void_config->{endpoints}) {
+				$generator->add_endpoints($self->void_config->{endpoints});
+			}
+			if ($self->void_config->{vocabularies}) {
+				$generator->add_vocabularies($self->void_config->{vocabularies});
+			}
+
+			# Do the stats and statements
+			$self->_voidmodel($generator->generate($file_model));
+			$self->last_etag($self->current_etag);
+		}
+
+		# Now start serializing.
 		my ($ct, $s) = $self->_negotiate($self->request->headers);
 		return $ct if ($ct->isa('Plack::Response')); # A hack to allow for the failed conneg case
 		my $body;
 		if ($s->isa('RDF::Trine::Serializer')) { # Then we just serialize since we have a serializer.
-			$body = $s->serialize_model_to_string($voidmodel);
+			$body = $s->serialize_model_to_string($self->_voidmodel);
 		} else {
 			# For (X)HTML, we need to do extra work
 			my $gen = RDF::RDFa::Generator->new( style => 'HTML::Pretty',
-															 title => 'VoID Description',
+															 title => $self->void_config->{pagetitle} || 'VoID Description',
 															 base => $self->base_uri,
 															 namespaces => $self->namespaces);
-			my $writer = HTML::HTML5::Writer->new( markup => 'xhtml', doctype => DOCTYPE_XHTML_RDFA );
-			$body = encode_utf8( $writer->document($gen->create_document($voidmodel)) );
+			my $markup = ($ct eq 'application/xhtml+xml') ? 'xhtml' : 'html';
+			my $writer = HTML::HTML5::Writer->new( charset => 'ascii', markup => $markup );
+			$body = $writer->document($gen->create_document($self->_voidmodel));
 		}
 		my $response = Plack::Response->new;
 		$response->status(200);
 		$response->headers->header('Vary' => join(", ", qw(Accept)));
-		$response->headers->header('ETag' => $self->etag);
+		$response->headers->header('ETag' => $self->last_etag);
 		$response->headers->content_type($ct);
-		$response->content($body);
+		$response->body(encode_utf8($body));
 		return $response;
 	} else {
 		return;
 	}
 }
 
+has _voidmodel => (is => 'rw', isa => 'RDF::Trine::Model', predicate => '_has_voidmodel', clearer => '_clear_voidmodel');
+
+
 =back
 
 
